/*
 *  thermal.h - Thermal device interface ($Revision: 1 $)
 *
 *  Copyright (C) 2006, 2007 Sujith Thomas <sujith.thomas@intel.com>

 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation;  version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Provide infrastructure from kernel to enable user-space
 * thermal management algorithm
 * Accepts registration of thermal devices(sensors).
 * Accepts registration for group of participants associated with this sensor
 * Accepts registration of extra attributes over the standard attributes.
 * Issues notification to registered drivers upon registration of new thermal
 * devices
 * Adds a 'Config' folder for user mode application to take over kernel mode
 * algo
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/list.h>

#define KELVIN_TO_CELSIUS(t) \
	(long)(((long)t-2732 >= 0) ? ((long)t-2732+5)/10 : ((long)t-2732-5)/10)
#define CELSIUS_TO_KELVIN(t)	((t*10) + 2732)

/* --------------------------------------------------------------------------
			Component Driver Interface
   -------------------------------------------------------------------------- */

struct thermal_device;

/*thermal device attributes*/
struct thermal_device_attribute {
	struct attribute attr;
	 ssize_t(*show) (struct thermal_device *td,
			 struct thermal_device_attribute *attr, char *buf);
	 ssize_t(*store) (struct thermal_device *td,
			  struct thermal_device_attribute *attr,
			  const char *buf, size_t count);
};

/*Helper macros for using THERMAL DEVICE attributes*/
#define THERMAL_DEVICE_ATTR(_name, _mode, _show, _store) \
struct thermal_device_attribute thermal_device_attr_##_name = {   \
	.attr = {  \
		.name = __stringify(_name), \
		.mode = _mode,       \
		.owner = THIS_MODULE,  \
	}, \
	.show = _show, \
	.store = _store, \
};

#define to_thermal_device_attr(_attr) \
	(container_of(_attr, \
		     struct thermal_device_attribute, \
		     attr))

#define to_thermal_device(obj) \
	container_of(obj, struct thermal_device, kobj)

#define THERMAL_ATTRIB_TEMPERATURE "temperature"
#define THERMAL_ATTRIB_CRITICAL "critical"
#define THERMAL_ATTRIB_HOT "hot"
#define THERMAL_ATTRIB_PASSIVE "passive"
#define THERMAL_ATTRIB_POLLING_FREQ "polling_freq"
#define THERMAL_ATTRIB_COOLING_MODE "cooling_mode"
#define THERMAL_ATTRIB_STATE "state"

/* The attributes which will be displayed for any thermal device */
struct thermal_device_ops {
	int (*thermal_get_temperature) (void *devdata,
					struct thermal_device_attribute *attr,
					int *temperature);
	int (*thermal_get_critical) (void *devdata,
				     struct thermal_device_attribute *attr,
				     int *temperature);
	int (*thermal_get_hot) (void *devdata,
				struct thermal_device_attribute *attr,
				int *temperature);
	int (*thermal_get_passive) (void *devdata,
				    struct thermal_device_attribute *attr,
				    int *temperature);
	int (*thermal_set_cooling_mode) (void *devdata,
					 struct thermal_device_attribute *attr,
					 int cooilng_mode);
	int (*thermal_get_polling_freq) (void *devdata,
					 struct thermal_device_attribute *attr,
					 int *polling_freq);
	int (*thermal_get_state) (void *devdata,
				  struct thermal_device_attribute *attr,
				  char *buf);
};

/* The participant which may be associated with a sensor
   For eg: CPU,Fan etc.. */
struct thermal_participant {
	char name[KOBJ_NAME_LEN];	/*Name of the participant*/
	struct thermal_group *group;	/*Group in which it belongs*/
	/*Pointer to kobj associated with default location in sysfs*/
	struct kobject *kobj;
};

/* The 'group' of devices associated with a sensor
   For Eg: passive_devices, active_devices */
struct thermal_group {
	struct list_head node;	/*Linked list management*/
	char name[KOBJ_NAME_LEN];	/*Name of the group*/
	/*Pointer to thermal device which contains this group*/
	struct thermal_device *td;
	int participant_count;	/*No:of participants in this group*/
	struct thermal_participant *participant;/*Array of participants*/
	struct kobject kobj;	/* Group's Kobject */
};

/* The thermal device information */
struct thermal_device {
	struct list_head node;	/* Linked list management*/
	struct thermal_device_ops *ops;	/*Callback routines*/
	char name[KOBJ_NAME_LEN];	/* Name of thermal device*/
	void *devdata;		/* Device's private data*/
	/* List of groups associated with this thermal device*/
	struct list_head group_node;
	struct kobject kobj;	/* Thermal device's Kobject*/
};

#define THERMAL_SYSFS_EVENT_ADD_TD 0x01
#define THERMAL_SYSFS_EVENT_ADD_GRP 0x02
#define THERMAL_SYSFS_EVENT_ADD_PART 0x04
#define THERMAL_SYSFS_EVENT_RMV_TD  0x08
#define THERMAL_SYSFS_EVENT_RMV_GRP 0x10
#define THERMAL_SYSFS_EVENT_RMV_PART 0x20

#define THERMAL_SYSFS_REPORT_EXISTING 0x0
#define THERMAL_SYSFS_NOREPORT_EXISTING 0x1

struct thermal_sysfs_event_listener {
	struct list_head node;
	/*Call back function (handler) upon receiving an event */
	int (*thermal_sysfs_event_handler) (int event_type
					    , void *sysfs_data
					    , void *private);
	int event_type;
	/* Event only from query_name will be notified*/
	char *query_name;
	/* Context of the registering driver*/
	void *private;

};

/* --------------------------------------------------------------------------
			Configuration Interface
   -------------------------------------------------------------------------- */

#define THERMAL_SYSFS_CONFIG "config"

#define THERMAL_MAX_MODE 2
enum thermal_userenabled { THERMAL_USER_DISABLED, THERMAL_USER_ENABLED };
enum thermal_mode { THERMAL_USERSPACE, THERMAL_KERNELSPACE };

struct thermal_config {
	u8 userenabled:1;/* 1 if user application is allowed to take over */
	u8 mode:1;		/* 0 if userspace algorithm has taken over */
	struct kobject kobj;	/*Syfs stuff */
};

/* --------------------------------------------------------------------------
			Registration functions
----------------------------------------------------------------------------- */

/*
 * thermal_device_register
 * ------------------------
 * Method for registering thermal devices(sensors) with sysfs
 * name: The name that should appear in sysfs
 * devdata : Device private context
 * ops : List of call back functions for various attributes
 */
struct thermal_device *thermal_device_register(const char *name,
					       void *devdata,
					       struct thermal_device_ops *ops);
/*
 * thermal_group_register
 * ------------------------
 * Method for registering groups such as ACPI _TZD with sysfs
 * name: The name that should appear in sysfs
 * td : Device under which this group is to be created
 * participant_count : No:of participants in this group
 * participants: Pointer to an array of participants
 */
int thermal_group_register(const char *name, struct thermal_device *td,
			   int participant_count,
			   struct thermal_participant *participant);

/*
 * thermal_attribute_register
 * ------------------------
 * Method for registering extra attributes with sysfs
 * td : Device under which attributes should be created
 * thermal_device_attrs : array of attributes to be added
 */
int thermal_attribute_register(struct thermal_device *td,
			       struct thermal_device_attribute
			       **thermal_device_attrs);

/*
 * thermal_event_register
 * ----------------------
 * Register for add/rmv of device/group/participant notification
 * handler: thermal_event_handler containing callback func pointer
 * report_type: Set this flag if callbacks needs to be invoked
 * for existing device/group/participants
 */
int thermal_event_register(struct thermal_sysfs_event_listener *listener,
			   unsigned int report_type);

/*
 * thermal_device_unregister
 * ------------------------
 * Method for unregistering thermal devices(sensors) with sysfs
 * td: Pointer to thermal_device
 */
int thermal_device_unregister(struct thermal_device *td);

/*
 * thermal_group_unregister
 * ------------------------
 * Method for unregistering groups within a thermal device
 * td: Pointer to thermal_device from where the group should be removed
 * name : Name of the group given during registration
 */
int thermal_group_unregister(struct thermal_device *td, const char *name);

/*
 * thermal_attribute_unregister
 * ------------------------
 * Method for unregistering extra attributes with sysfs
 * td : Device under which attributes should be removed
 * thermal_device_attrs : array of attributes to be removed
 */
int thermal_attribute_unregister(struct thermal_device *td,
				  struct thermal_device_attribute
				  **thermal_device_attrs);
/*
 * thermal_event_unregister
 * ----------------------
 * UnRegister for add/rmv of device/group/participant notification
 * handler: thermal_event_handler containing callback func pointer
 * report_type: Set this flag if callbacks needs to be invoked
 * for existing device/group/participants
 */
int thermal_event_unregister(struct thermal_sysfs_event_listener *listener,
			     unsigned int report_type);
/*
 * thermal_get_algo_mode
 * ----------------------
 * Method to know whether the user mode application has taken over
 */
int thermal_get_algo_mode(void);

/*
 * thermal_sysfs_generate_event
 * ---------------------------
 * Drivers managing thermal devices can invoke this method to notify
 * user applications about thermal events
 */
int thermal_sysfs_generate_event(struct thermal_device *td,
				 enum kobject_action action);

/*
 * thermal_set_userenabled
 * -----------------------
 * Interface function for platform sensor driver to disble userspace algo
 * ue: enable / disable userspace algo based on BIOS configuration
 */
int thermal_set_userenabled(enum thermal_userenabled um);
